class administrativeGender:
    contents=[{'code': 'F',
            'display': 'Female',
            'system': 'http://hl7.org/fhir/v3/AdministrativeGender'},
        {'code': 'M',
            'display': 'Male',
            'system': 'http://hl7.org/fhir/v3/AdministrativeGender'},
        {'code': 'UN',
            'display': 'Undifferentiated',
            'system': 'http://hl7.org/fhir/v3/AdministrativeGender'}]

__all__=['administrativeGender']
