class maritalStatus:
    contents=[
        {'code': 'A',
            'display': 'Annulled',
            'system': 'http://hl7.org/fhir/v3/MaritalStatus'},
        {'code': 'D',
            'display': 'Divorced',
            'system': 'http://hl7.org/fhir/v3/MaritalStatus'},
        {'code': 'I',
            'display': 'Interlocutory',
            'system': 'http://hl7.org/fhir/v3/MaritalStatus'},
        {'code': 'L',
            'display': 'Legally separated',
            'system': 'http://hl7.org/fhir/v3/MaritalStatus'},
        {'code': 'M',
            'display': 'Married',
            'system': 'http://hl7.org/fhir/v3/MaritalStatus'},
        {'code': 'P',
            'display': 'Polygamous',
            'system': 'http://hl7.org/fhir/v3/MaritalStatus'},
        {'code': 'S',
            'display': 'Never married',
            'system': 'http://hl7.org/fhir/v3/MaritalStatus'},
        {'code': 'T',
            'display': 'Domestic partner',
            'system': 'http://hl7.org/fhir/v3/MaritalStatus'},
        {'code': 'W',
            'display': 'Widowed',
            'system': 'http://hl7.org/fhir/v3/MaritalStatus'}
        ]

__all__=['maritalStatus']
