
STATUS_CODES={
        100:    "Continue",
        102:    "Processing",
        200:    "Ok",
        201:    "Created",
        204:    "No Content",
        207:    "Multi-Status",
        400:    "Bad Request",
        403:    "Forbidden",
        404:    "Not Found",
        405:    "Method Not Allowed",
        409:    "Conflict",
        412:    "Precondition failed",
        422:    "Unprocessable Entity",
        423:    "Locked",
        424:    "Failed Dependency",
        502:    "Bad Gateway",
        507:    "Insufficient Storage"
}
