class immunizationSite:
    contents=[
        { 'code': 'LVL', 'display': 'left vastus lateralis' },
        { 'code': 'RVL', 'display': 'right vastus lateralis' },
        { 'code': 'LD', 'display': 'left deltoid' },
        { 'code': 'RD', 'display': 'right deltoid' },
        { 'code': 'LALT', 'display': 'left anterolateral fat of thigh' },
        { 'code': 'RALT', 'display': 'right anterolateral fat of thigh' },
        { 'code': 'LPUA', 'display': 'left posterolateral fat of upper arm' },
        { 'code': 'RPUA', 'display': 'right posterolateral fat of upper arm' },
        { 'code': 'LFA', 'display': 'left fore arm' },
        { 'code': 'RFA', 'display': 'right fore arm' }]

__all__=['immunizationSite']
